from ipywidgets import register
from traitlets import Bool, Float, Instance, Integer, Unicode

from aipython.probGraphicalModels import Belief_network
from aipython.probVE import VE

from ... import __version__
from ..stepdomwidget import ReturnableThread, StepDOMWidget
from .bayesjsonbridge import bayes_problem_to_json, json_to_bayes_problem


@register
class Displayable(StepDOMWidget):
    _view_name = Unicode('BayesVisualizer').tag(sync=True)
    _model_name = Unicode('BayesVisualizerModel').tag(sync=True)
    _view_module = Unicode('aispace2').tag(sync=True)
    _model_module = Unicode('aispace2').tag(sync=True)
    _view_module_version = Unicode(__version__).tag(sync=True)
    _model_module_version = Unicode(__version__).tag(sync=True)

    graph = Instance(klass=Belief_network, allow_none=True)\
        .tag(sync=True, from_json=json_to_bayes_problem, to_json=bayes_problem_to_json)

    decimal_place = Integer(2).tag(sync=True)

    probCalculator = VE()

    def __init__(self):
        super().__init__()
        self.graph = self.problem
        self.probCalculator = VE(self.problem)

    def _validate_line_width(self, proposal):
        """Cap line_width at a minimum value."""
        line_width = proposal['value']
        return min(1, line_width)

    def handle_custom_msgs(self, _, content, buffers=None):
        super().handle_custom_msgs(None, content, buffers)
        event = content.get('event', '')

        # Receive msg from frontend
        if event == "node:query":
            name = content.get('name')
            evidences = content.get('evidences')
            self._query_node(name, evidences)
        elif event == 'initial_render':
            queued_func = getattr(self, '_queued_func', None)

            # Run queued function after we know the frontend view exists
            if queued_func:
                func = queued_func['func']
                args = queued_func['args']
                kwargs = queued_func['kwargs']
                self._previously_rendered = True
                self._thread = ReturnableThread(
                    target=func, args=args, kwargs=kwargs)
                self._thread.start()

        elif event == 'update_sleep_time':
            self.sleep_time = content.get('sleepTime')

    def _query_node(self, name, evidences):
        name_to_node = {v.name: v for v in self.graph.variables}
        node = name_to_node[name]
        evidence_list = {name_to_node[e['name']]: e['value'] for e in evidences}
        reply = self.probCalculator.query(node, evidence_list)
        self.send({
            "action": "query",
            "name": name,
            "prob": reply
        })
